--- ogg123.c.dec15_2k1_beta	Wed Nov 28 06:47:16 2001
+++ ogg123.c	Sat Dec 15 21:54:46 2001
@@ -1,5 +1,6 @@
 /* ogg123.c by Kenneth Arnold <kcarnold@arnoldnet.net> */
 /* Modified to use libao by Stan Seibert <volsung@asu.edu> */
+/* Modified to accept range by Maciej Zenczykowski <maciej_zenczykowski@iname.com> */
 
 /********************************************************************
  *                                                                  *
@@ -68,6 +69,7 @@
     {"device", required_argument, 0, 'd'},
     {"file", required_argument, 0, 'f'},
     {"skip", required_argument, 0, 'k'},
+    {"range", required_argument, 0, 'r'},
     {"device-option", required_argument, 0, 'o'},
     {"verbose", no_argument, 0, 'v'},
     {"quiet", no_argument, 0, 'q'},
@@ -102,6 +104,8 @@
 	    "  -f, --file=filename  Set the output filename for a previously\n"
 	    "      specified file device (with -d).\n"
 	    "  -k n, --skip n  Skip the first 'n' seconds\n"
+	    "  -r ..., --range [[hh:]mm:]ss[.hh]-[[hh:]mm:]ss[.hh]\n"
+	    "      Play the given range.\n"
 	    "  -o, --device-option=k:v passes special option k with value\n"
 	    "      v to previously specified device (with -d).  See\n"
 	    "      man page for more info.\n"
@@ -125,12 +129,18 @@
     ao_info *info;
     int temp_driver_id = -1;
     devices_t *current;
+    /* the following are used for the --range option only */
+    double f, ff;
+    char * pc;
+    int decimal_digits;
+    int state;
     
     opt.read_file = NULL;
     opt.shuffle = 0;
     opt.verbose = 0;
     opt.quiet = 0;
     opt.seekpos = 0;
+    opt.endpos = -1; /* means ignore */
     opt.instream = NULL;
     opt.outdevices = NULL;
     opt.buffer_size = 0;
@@ -138,7 +148,7 @@
 
     ao_initialize();
 
-    while (-1 != (ret = getopt_long(argc, argv, "b:d:f:hl:k:o:qvVz",
+    while (-1 != (ret = getopt_long(argc, argv, "b:d:f:hl:k:r:o:qvVz",
 				    long_options, &option_index))) {
 	switch (ret) {
 	case 0:
@@ -181,6 +191,104 @@
 	    break;
 	case 'k':
 	    opt.seekpos = atof(optarg);
+	    opt.endpos = -1;
+	    break;
+	case 'r':
+	    /* OK: what we want to support is: [[[[hh]:]mm]:]ss[.[hh]]-[[[[hh]:]mm]:]ss[.[hh]] for a range
+	       or alternatively as above but with + instead of minus for a beginning and length...
+	       assume no - or + means just a beginning location. */
+	    /* fprintf(stderr, "Debug: '%s'.\n", optarg); */
+	    opt.seekpos = 0;
+	    opt.endpos = -1;
+	    
+	    pc = optarg;
+	    f = 0;
+	    ff = 0;
+	    decimal_digits = -1;
+	    state = 0;
+	    
+	    do {
+	        switch (*pc) {
+		    case '0': case '1': case '2': case '3': case '4':
+		    case '5': case '6': case '7': case '8': case '9':
+			if (decimal_digits >= 0) decimal_digits++;
+			ff *= 10;
+			ff += *pc - '0';
+			break;
+		    case ':':
+			f += ff;
+			ff = 0;
+			f *= 60;
+			break;
+		    case '.':
+			f += ff;
+			ff = 0;
+			decimal_digits = 0;
+			break;
+		    case '-':
+		    case '+':
+		    case 0:
+			while (decimal_digits-- > 0) ff /= 10;
+			decimal_digits = -1;
+			f += ff;
+			ff = 0;
+			
+			switch (*pc) {
+			    case '-':
+				if (state) {
+				    fprintf(stderr, "Error: Too many parameters to range...\n\n");
+				    usage();
+				    exit(1);
+				};
+				opt.seekpos = f;
+				state = 1;
+				break;
+			    case '+':
+				if (state) {
+				    fprintf(stderr, "Error: Too many parameters to range...\n");
+				    usage();
+				    exit(1);
+				};
+				opt.seekpos = f;
+				state = 2;
+				break;
+			    case 0:
+				switch (state) {
+				    case 0:
+					opt.seekpos = f;
+					opt.endpos = -1;
+					break;
+				    case 1:
+					opt.endpos = f;
+					break;
+				    case 2:
+					opt.endpos = opt.seekpos + f;
+					break;
+				}
+				break;
+			}
+			
+			f = 0;
+			
+			break;
+		    default:
+			fprintf(stderr, "Error: Unable to parse: '%c' in range description '%s'\n\n", *pc, optarg);
+			usage();
+			exit(1);			
+	        }
+	    } while (*pc++);	    
+	    
+	    if (opt.endpos >= 0) {
+	    	if (opt.endpos < opt.seekpos) {
+		    fprintf(stderr, "Error: Range beginning: %0.2lfs is after range ending: %0.2lfs.\n\n");
+		    usage();
+		    exit(1);
+		};
+		fprintf(stderr, "Range selected: %0.2lfs to %0.2lfs.\n", opt.seekpos, opt.endpos);
+	    } else {
+		fprintf(stderr, "Range selected: %0.2lfs till end.\n", opt.seekpos);
+	    };
+	    
 	    break;
 	case 'l':
 	    opt.delay = atoi(optarg);
@@ -312,6 +420,7 @@
     double t_sec = 0, c_sec = 0, r_sec = 0;
     int is_big_endian = ao_is_big_endian();
     double realseekpos = opt.seekpos;
+    double realendpos = opt.endpos;
 
     /* Junk left over from the failed info struct */
     double u_time, u_pos;
@@ -456,6 +565,10 @@
 	     * right to the end when we seek it will go to the beginning of the song */
 	    realseekpos = ov_time_total(&vf, -1) - 0.01;
 
+	if ((realendpos > ov_time_total(&vf, -1)) || (realendpos < 0))
+	    /* If we're out of range set it to the end, same if nothing was specified... */
+	    realendpos = ov_time_total(&vf, -1);
+
 	if (realseekpos > 0)
 	    ov_time_seek(&vf, realseekpos);
 
@@ -504,6 +617,8 @@
 		  }
 		else
 		  devices_write(convbuffer, ret, opt.outdevices);
+
+		if (ov_time_tell(&vf) >= realendpos) eof = eos = 1; /* simulate end of file/stream */
 		
 		if (opt.verbose > 0) {
 		    if (ov_seekable (&vf)) {
